/**
 * @author Eugene Turkov
 * @author <a href="mailto:eugene.turkov@nasa.gov">eugene.turkov@nasa.gov</a>
 * @version: 1.0
 * @since SDK1.5
 * 
 * NASA Ames Research Center Computational Sciences Division Intelligent Data
 * Understanding Group (IDU) Moffett Field, CA 94035
 */

package turkov.util;

import java.util.Random;

public abstract class TUCollectionUtil
{
	public static int getNumberOfOccurences(double value, double[] collection)
	{
		int count = 0;
		for (int i = 0; i < collection.length; i++)
		{
			if (collection[i] == value)
				count++;
		}

		return count;
	}

	/**
	 Using matlab's idea behind "randperm" this method returns a subset of items
	 from the entire data set of size subsetSize.
	 
	 @param data the collection of items from which we are to choose subsetSize
	 items to return.
	 @param subsetSize the size of the desired subset of data.
	 @return a subset of data of size subsetSize.
	 */
	public static Object[] getRandomSubset(Object[] data, int subsetSize,int randomSeed)
	{
		int numIterations = (int) Math.ceil((double) data.length / (double) 2);
		Object[] tempData = new Object[data.length];
		Object[] subset = new Object[subsetSize];

		for (int i = 0; i < data.length; i++)
			tempData[i] = data[i];

		Random random = new Random(randomSeed);
		for (int i = 0; i < numIterations; i++)
		{
			int firstIndex = random.nextInt(tempData.length);
			int secondIndex = random.nextInt(tempData.length);
			Object tempValue = tempData[firstIndex];
			tempData[firstIndex] = tempData[secondIndex];
			tempData[secondIndex] = tempValue;
		}

		for (int i = 0; i < subsetSize; i++)
			subset[i] = tempData[i];

		return subset;
	}

	public static int getPosition(Object object, Object[] collection,
			int startIndex)
	{
		//System.out.println("start index: " + startIndex);
		//System.out.println("objet: " + object);
		int result = -1;
		for (int i = startIndex; i < collection.length; i++)
		{
			if (collection[i].equals(object))
			{
				result = i;
				break;
			}
		}

		//System.out.println("found position of " + object + " at " + result);
		return result;
	}

	public static void main(String[] args)
	{

		String[] longestCommonSubsequenceTemp = "2 69 69".split(" ");
		Integer[] longestCommonSubsequence = new Integer[longestCommonSubsequenceTemp.length];

		for (int i = 0; i < longestCommonSubsequenceTemp.length; i++)
			longestCommonSubsequence[i] = Integer
					.valueOf(longestCommonSubsequenceTemp[i]);

		String[] outlierSequenceTemp = "2 69 69 65".split(" ");
		Integer[] outlierSequence = new Integer[outlierSequenceTemp.length];

		for (int i = 0; i < outlierSequenceTemp.length; i++)
			outlierSequence[i] = Integer.valueOf(outlierSequenceTemp[i]);

		int lcsToOutlierSequenceIndices[] = new int[longestCommonSubsequence.length];
		// second, determine outlierSequenceToLcsMapping
		int currentPosition = 0;

		currentPosition = 0;
		for (int j = 0; j < longestCommonSubsequence.length; j++)
		{
			lcsToOutlierSequenceIndices[j] = TUCollectionUtil.getPosition(
					longestCommonSubsequence[j], outlierSequence,
					currentPosition);
			System.out.println(j + " --> " + lcsToOutlierSequenceIndices[j]);
			System.out.println("*" + longestCommonSubsequence[j] + " --> "
					+ outlierSequence[lcsToOutlierSequenceIndices[j]]);

			//System.out.println("the outlier sequence is: " + TUStringBuilder.concatenate(outlierSequence));
			//System.out.println("the lcs is: " + TUStringBuilder.concatenate(longestCommonSubsequence));
			/* 
			 sb.append(j + " --> " + lcsToOutlierSequenceIndices[j]);
			 sb.append("the outlier sequence is: " + TUStringBuilder.concatenate(outlierSequence));
			 sb.append(System.getProperty("line.separator"));
			 */

			currentPosition = lcsToOutlierSequenceIndices[j] + 1;
		}
	}
}
